{% load mezzanine_tags %}
<div class="panel panel-default user-panel">
<div class="panel-body">
{% ifinstalled cartridge.shop %}
{% include "shop/includes/user_panel.html" %}
{% endifinstalled %}
{% ifinstalled mezzanine.accounts %}
{% ifinstalled cartridge.shop %}<br>{% endifinstalled %}
{% include "accounts/includes/user_panel.html" %}
{% endifinstalled %}
</div>
</div>
