"""
Provides the main structure of a Mezzanine site with a hierarchical tree
of pages, each subclassing the Page model to create a content structure.
"""
from __future__ import unicode_literals

from mezzanine import __version__  # noqa


default_app_config = 'mezzanine.pages.apps.PagesConfig'
