"""
Provides abstract models and admin features used throughout the various
Mezzanine apps.
"""
from __future__ import unicode_literals

from mezzanine import __version__  # noqa


default_app_config = 'mezzanine.core.apps.CoreConfig'
