<!doctype html>
<html lang="{{ LANGUAGE_CODE }}"{% if LANGUAGE_BIDI %} dir="rtl"{% endif %}>
{% load pages_tags mezzanine_tags i18n staticfiles %}
<head>

<meta http-equiv="Content-type" content="text/html; charset=utf-8" />
<meta name="viewport" content="initial-scale=1, maximum-scale=1">
<meta name="keywords" content="{% block meta_keywords %}{% endblock %}" />
<meta name="description" content="{% block meta_description %}{% endblock %}" />
<title>{% block meta_title %}{% endblock %}{% if settings.SITE_TITLE %} | {{ settings.SITE_TITLE }}{% endif %}</title>
<link rel="shortcut icon" href="{% static "img/favicon.ico" %}">
<link rel="apple-touch-icon" href="">

{% ifinstalled mezzanine.blog %}
<link rel="alternate" type="application/rss+xml" title="RSS" href="{% url "blog_post_feed" "rss" %}" />
<link rel="alternate" type="application/atom+xml" title="Atom" href="{% url "blog_post_feed" "atom" %}" />
{% endifinstalled %}

{% compress css %}
<link rel="stylesheet" href="{% static "css/global-mobile.css" %}">
<link rel="stylesheet" href="{% static "css/jquery.mobile-1.2.1.min.css" %}" />
{% if LANGUAGE_BIDI %}
<link rel="stylesheet" href="{% static "css/mobile.rtl.css" %}">
{% endif %}
{% block extra_css %}{% endblock %}
{% endcompress %}

{% block extra_head %}{% endblock %}

</head>
<body id="{% block body_id %}{% endblock %}">
<div data-role="page" data-theme="d">

    <div data-role="header">
        <h1>{% block title %}{% endblock %}</h1>
        <a class="home" data-role="button" data-icon="star" href="{% url "home" %}">{% trans "Home" %}</a>
    </div>

    {% page_menu "pages/menus/mobile.html" %}

    <div data-role="content">
        {% block main %}{% endblock %}
    </div>

    <div data-role="footer">
        <div data-role="content" align="center">
        	{% include "mobile/includes/search.html" %}
            <p>{% trans "Powered by" %} <a href="http://mezzanine.jupo.org" rel="external">Mezzanine</a></p>
            <p><a href="{% url "set_device" "default" %}?next={{ request.path }}" rel="external">{% trans "View Full Site" %}</a></p>
        </div>
    </div>

</div>

{% compress js %}
{% block footer_js %}
<script src="{% static "mezzanine/js/"|add:settings.JQUERY_FILENAME %}"></script>
<script src="{% static "js/jquery.mobile-1.2.1.min.js" %}"></script>
{% include "mobile/includes/footer_scripts.html" %}
{% endblock %}
{% endcompress %}

</body>
</html>

